/*************************************************************
**                                                          **
**  TWF reader                                              **
**  Reads a TWF file calling necessary callbacks for        **
**  drawing and saving file into memory.                    **
**  by Mark Stroetzel Glasberg - Tecgraf, PUC-RIO 1999      **
**                                                          **
// Falta implementar Mark
*************************************************************/

#ifndef TWFR_READ_
#define TWFR_READ_

/*************************************************************
**  Configuracoes globais                                   **
**                                                          **
*************************************************************/
#ifndef __TWFRGLOBALH__
#define FileInfo void                        /* File information */

/*************************************************************
**  Estruturas exportadas                                   **
**                                                          **
*************************************************************/
typedef struct struct_Face {
  int numedges;
  int *edges;
  int nholes; 
  int *holes;
} Face;

typedef Edge (void);

#endif

typedef struct struct_TwfPoint  {
  double x;
  double y;
} TwfPoint;

typedef int (*Tcallback)(FileInfo *, ...);   /* Callback         */

#define DO_NOT_RENDER              /* Does not pass parameter to OPENGL         */

#define TWFR_MAX_ID_SIZE       40   /* Maximum string length of a command name   */
#define TWFR_NUM_MAX_IDS       500  /* Maximum number of TWF commands            */

/*************************************************************
**  Defines exportadas                                      **
**                                                          **
*************************************************************/
#define TWFR_NUM_MAX_SHAPES                256
#define TWFR_NUM_MAX_IMAGES                256

/*************************************************************
**  Funcoes exportadas                                      **
**                                                          **
*************************************************************/
FileInfo *twfread(char *filename, void *drawing, void *userdata, int render, char *outtextfile);
void      twfclosefile(FileInfo **finf); 
void      twfregister(int id, void *cb);
char      *twfgeterror(FileInfo *info);

/*************************************************************
**  Funcoes exportadas                                      **
**  Que leem a estrutura                                    **
*************************************************************/
int       twfisfileset(FileInfo *info);
void      *twfgetdrawing(FileInfo *info);
void      *twfgetuserdata(FileInfo *info);
int       twfgetcompilationtime(FileInfo *info);
int       twfgetfillcolormode(FileInfo *info);
double    twfgetcolor(FileInfo *info, int type, int channel);
int       twfgetimagestretchmode(FileInfo *inf);
double    twfgetpoint(FileInfo *info, int type, int axis);
int       twfgettexturefillid(FileInfo *info);
int       twfgetnumlayers(FileInfo *info);
int       twfgetnumimages(FileInfo *info);
int       twfgetnumshapes(FileInfo *info);
int       twfgettypeface(FileInfo *info);
int       twfgetnormalalignment(FileInfo *info);
int       twfgettextsize(FileInfo *info);
double    twfgettextdir(FileInfo *info);
int       twfgettextstyle(FileInfo *info);
void      twfgeturpoint(FileInfo *info, double *x, double *y);
void      twfgetllpoint(FileInfo *info, double *x, double *y);
int       twfgetmarktype(FileInfo *info);
int       twfgetmarksize(FileInfo *info);
void      twfgetlinestyle (FileInfo *info, short *joint, short *begin, short *end, short *width);
int       twfgetlinedashes(FileInfo *info);
int       twfgetblendmode (FileInfo *info);
Face      *twfgetface(FileInfo *inf, long faceid);
long      twfgetcurrentfaceid(FileInfo *inf);
long      twfgetcurrentedgeid(FileInfo *inf);
Edge      *twfgetedge(FileInfo *info, long edgenum);
int       findedgenumsegs(Edge *e);
int       twfgetedgenumsegs(Edge *e);
int       twfgetedgepoint(Edge *e, int i, int j, TwfPoint *a);


/*************************************************************
**  Lista de identificadores de funcoes de Callbacks        **
**  que podem ser registradas.                              **
**                                                          **
*************************************************************/
enum {
 TWFR_setBlendMode	         = 57,  /* int funcao(FileInfo *inf, unsigned char mode) */
 TWFR_ellipse	               = 58,  /* int funcao(FileInfo *inf, double cx, double cy, double w, double h, fillelement ) */
 TWFR_beginLayer             = 59,  /* int funcao(FileInfo *inf, char *name, unsigned char vis, unsigned char detect ) */
 TWFR_close                  = 60,  /* int funcao(FileInfo *inf) */
 TWFR_setLineDashes          = 71,  /* int funcao(FileInfo *inf, unsigned short dash ); */
 TWFR_setLineStyle           = 72,  /* int funcao(FileInfo *inf, int joint, int begin, int end, unsigned char weight ); */
 TWFR_box                    = 73,  /* int funcao(FileInfo *inf, double x, double y, double w, double h, int fillelement ); */
 TWFR_line                   = 74,  /* int funcao(FileInfo *inf, double x1, double y1, double x2, double y2); */
 TWFR_beginSetShape          = 75,  /* int funcao(FileInfo *inf, double w, double h, unsigned long *id ); */
 TWFR_endSetShape            = 76,  /* int funcao(FileInfo *inf); */
 TWFR_begin                  = 77,  /* int funcao(FileInfo *inf, int type, double x, double y, int fillelement ); */
 TWFR_vertex                 = 78,  /* int funcao(FileInfo *inf,  double x, double y ); */
 TWFR_end                    = 79,  /* int funcao(FileInfo *inf); */
 TWFR_circle                 = 80,  /* int funcao(FileInfo *inf, double x, double y, double radius, int fillelement ); */
 TWFR_arc                    = 81,  /* int funcao(FileInfo *inf, double x, double y, double w, double h, double start, double end, int fillelement ); */
 TWFR_stampShape             = 83,  /* int funcao(FileInfo *inf, unsigned long id, double x, double y); */
 TWFR_shape                  = 82,  /* int funcao(FileInfo *inf, unsigned long id, double x,  double y, double rotation, double w, double h ); */
 TWFR_extension              = 85,  /* int funcao(FileInfo *inf, char *parameter, char *value ); */
 TWFR_beginObject            = 86,  /* int funcao(FileInfo *inf, char *name ); */
 TWFR_endObject              = 87,  /* int funcao(FileInfo *inf); */
 TWFR_text                   = 88,  /* int funcao(FileInfo *inf, double x, double y, char *text );  */
 TWFR_setImage               = 89,  /* int funcao(FileInfo *inf, unsigned short w, unsigned short h, float *colors, unsigned long *id ); */
 TWFR_fullImage              = 90,  /* int funcao(FileInfo *inf, unsigned long id, double x, double y, double w, double h ); */
 TWFR_image                  = 91,  /* int funcao(FileInfo *inf, int image_id, double srcllx, double srclly, double srcurx, double srcury, double dstllx, double dstlly, double dsturx, double dstury); */
 TWFR_mark                   = 92,  /* int funcao(FileInfo *inf, double location.x, double location.y); */
 TWFR_drawedge               = 93,  /* int funcao(FileInfo *inf, long edgenum); */
 TWFR_fillface               = 94,  /* int funcao(FileInfo *inf, long fillfacenum); */
} Callbacks;


/* Primitive types for TWFR_begin */
enum {
  TWFR_BEGIN_POLYLINE,
  TWFR_BEGIN_POLYGON,
  TWFR_BEGIN_CUBIC,
};

enum {
  TWFR_GRAD,
  TWFR_RGBA,
  TWFR_IMAGE,
  TWFR_IMAGE_STRETCHED,
};

enum {
  TWFR_LINE_COLOR,
  TWFR_FILL_COLOR,
  TWFR_MARK_COLOR,
  TWFR_BG_COLOR,
  TWFR_GRAD_COLOR0,
  TWFR_GRAD_COLOR1,
  TWFR_TEXT_COLOR,
} ColorType;

enum {
  TWFR_CHANNEL_R,
  TWFR_CHANNEL_G,
  TWFR_CHANNEL_B,
  TWFR_CHANNEL_A,
} Channel;

enum {
  TWFR_NO  = 0,
  TWFR_YES = 1,
};

enum { 
  TWFR_NEAREST  = 0,
  TWFR_BILINEAR = 1, 
} StretchMode;

enum {
  TWFR_STRETCHLL,
  TWFR_STRETCHUR,
  TWFR_GRADPOINT0,
  TWFR_GRADPOINT1,
} Pointtypes;


enum {
  TWFR_X,
  TWFR_Y,
} Axis;


enum {
  TWFR_Plain,
  TWFR_Bold,
  TWFR_Italic,
  TWFR_Bold_Italic,
} FontStyles;


enum {
  TWFR_BLEND_Clear,
  TWFR_BLEND_Source,
  TWFR_BLEND_Source_Over_Destination,
  TWFR_BLEND_Destination_Over_Source,
  TWFR_BLEND_Source_In_Destination,
  TWFR_BLEND_Destination_In_Source,
  TWFR_BLEND_Source_Held_Out_By_Destination,
  TWFR_BLEND_Destination_Held_Out_By_Source,
} BlendStyle;


enum {

  /* Global         */
  TWFR_ERROR   = -4,
  TWFR_DEFAULT = 1,

  /* Memory control */
  TWFR_FREE_OBJECT, 
  TWFR_GIVE_OBJECT,

} ReturnTypes;

#endif